@@ -31,6 +31,8 @@ using System.IO;
 using System.Text;
 using System.Xml;
 
+using Mono.Posix;
+
 namespace Beagle.Util {
 
 	public class StringFu {
@@ -60,20 +62,21 @@ namespace Beagle.Util {
 			string date = null, time = null;
 
 			if (sinceToday.TotalDays <= 0)
-				date = "Today";
+				date = Catalog.GetString ("Today");
 			else if (sinceToday.TotalDays < 1)
-				date = "Yesterday";
+				date = Catalog.GetString ("Yesterday");
 			else if (today.Year == dt.Year)
-				date = dt.ToString ("MMM d");
+				date = dt.ToString (Catalog.GetString ("MMM d"));
 			else
-				date = dt.ToString ("MMM d, yyyy");
+				date = dt.ToString (Catalog.GetString ("MMM d, yyyy"));
 			
-			time = dt.ToString ("h:mm tt");
+			time = dt.ToString (Catalog.GetString ("h:mm tt"));
 
 			string fuzzy;
 
 			if (date != null && time != null)
-				fuzzy = date + ", " + time;
+				/* Translators: {0} is a date (e.g. 'Today' or 'Apr 23'), {1} is the time */
+				fuzzy = String.Format (Catalog.GetString ("{0}, {1}"), date, time);
 			else if (date != null)
 				fuzzy = date;
 			else 
@@ -88,19 +91,23 @@ namespace Beagle.Util {
 			string short_time = date.ToShortTimeString ();
 
 			if (date.Year == now.Year) {
-				if (date.DayOfYear == now.DayOfYear)
-					return String.Format ("Today, {0}", short_time);
-				else if (date.DayOfYear == now.DayOfYear - 1)
-					return String.Format ("Yesterday, {0}", short_time);
-				else if (date.DayOfYear > now.DayOfYear - 6)
-					return String.Format ("{0} days ago, {1}",
+				if (date.DayOfYear == now.DayOfYear) {
+					/* To translators: {0} is the time of the day, eg. 13:45 */
+					return String.Format (Catalog.GetString ("Today, {0}"), short_time);
+				} else if (date.DayOfYear == now.DayOfYear - 1) {
+					/* To translators: {0} is the time of the day, eg. 13:45 */
+					return String.Format (Catalog.GetString ("Yesterday, {0}"), short_time);
+				} else if (date.DayOfYear > now.DayOfYear - 6) {
+					/* To translators: {0} is the number of days that have passed, {1} is the time of the day, eg. 13:45 */
+					return String.Format (Catalog.GetString ("{0} days ago, {1}"),
 							      now.DayOfYear - date.DayOfYear,
 							      short_time);
-				else
-					return date.ToString ("MMMM d, h:mm tt");
+				} else {
+					return date.ToString (Catalog.GetString ("MMMM d, h:mm tt"));
+				}
 			}
 
-			return date.ToString ("MMMM d yyyy, h:mm tt");
+			return date.ToString (Catalog.GetString ("MMMM d yyyy, h:mm tt"));
 		}
 		
 		public static string DurationToPrettyString (DateTime end_time, DateTime start_time)
@@ -110,20 +117,14 @@ namespace Beagle.Util {
 			string span_str = ""; 
 
 			if (span.Hours > 0) {
-				if (span.Hours == 1)
-					span_str = "1 hour";
-				else
-					span_str = String.Format ("{0} hours", span.Hours);
-
+				span_str = String.Format (Catalog.GetPluralString ("{0} hour", "{0} hours", span.Hours), span.Hours);
+					
 				if (span.Minutes > 0)
 					span_str += ", ";
 			}
 
 			if (span.Minutes > 0) {
-				if (span.Minutes == 1)
-					span_str += "1 minute";
-				else
-					span_str += String.Format ("{0} minutes", span.Minutes);
+				span_str += String.Format (Catalog.GetPluralString ("{0} minute", "{0} minutes", span.Minutes), span.Minutes);
 			}
 					
 			
@@ -138,12 +139,15 @@ namespace Beagle.Util {
 				return "*BadLength*";
 
 			if (len < 1024)
-				return String.Format ("{0} bytes", len);
+				/* Translators: {0} is a file size in bytes */
+				return String.Format (Catalog.GetString ("{0} bytes"), len);
 
 			if (len < oneMb)
-				return String.Format ("{0:0.0} kb", len/(double)1024);
+				/* Translators: {0} is a file size in kilobytes */
+				return String.Format (Catalog.GetString ("{0:0.0} KB"), len/(double)1024);
 
-			return String.Format ("{0:0.0} Mb", len/(double)oneMb);
+			/* Translators: {0} is a file size in megabytes */
+			return String.Format (Catalog.GetString ("{0:0.0} MB"), len/(double)oneMb);
 		}
 
 		// FIXME: This is pretty inefficient
@@ -271,23 +275,28 @@ namespace Beagle.Util {
 			return false;
 		}
 
-		static char[] CharsToQuote = { ';', '?', ':', '@', '&', '=', '$', ',', '#' };
+		static char[] CharsToQuote = { ';', '?', ':', '@', '&', '=', '$', ',', '#', '%', '"', ' ' };
+
+		static public string HexEscape (string str)
+		{
+			StringBuilder builder = new StringBuilder ();
+			int i;
+
+			while ((i = str.IndexOfAny (CharsToQuote)) != -1) {
+				if (i > 0)
+					builder.Append (str.Substring (0, i));
+				builder.Append (Uri.HexEscape (str [i]));
+				str = str.Substring (i+1);
+			}
+			builder.Append (str);
+
+			return builder.ToString ();
+		}
+
 		static public string PathToQuotedFileUri (string path)
 		{
-
-			StringBuilder builder = new StringBuilder (Uri.UriSchemeFile + Uri.SchemeDelimiter);
-			int i;
 			path = Path.GetFullPath (path);
-			while ((i = path.IndexOfAny (CharsToQuote)) != -1) {
-				if (i > 0)
-					builder.Append (path.Substring (0, i));
-				builder.Append (Uri.HexEscape (path [i]));
-				path = path.Substring (i+1);
-			}
-			builder.Append (path);
-			
-			return builder.ToString ();
-
+			return Uri.UriSchemeFile + Uri.SchemeDelimiter + HexEscape (path);
 		}
 
 		// These strings should never be exposed to the user.
@@ -400,6 +409,21 @@ namespace Beagle.Util {
                         return s;
 		}
 
+		static public string CleanupInvalidXmlCharacters (string str)
+		{
+			int len = str.Length;
+			char[] char_array = new char[len];
+
+			for (int i = 0; i < len; i++) {
+				if (IsInvalid (str[i]))
+					char_array[i] = ' ';
+				else
+					char_array[i] = str[i];
+			}
+
+			return new string (char_array);
+		}
+		
 		static public int CountWords (string str, int max_words)
 		{
 			if (str == null)
