@@ -35,10 +35,16 @@ using Gtk;
 
 using Beagle.Util;
 
+#if ENABLE_WEBSERVICES
+using Beagle.WebService;
+using MA=Mono.ASPNET;
+#endif
 
 namespace Beagle.Daemon {
 	class BeagleDaemon {
 
+		public static Thread MainLoopThread = null;
+
 		private static FactoryImpl factory = null;
 
 #if ENABLE_NETWORK
@@ -152,19 +158,31 @@ namespace Beagle.Daemon {
 		{
 			while (! Shutdown.ShutdownRequested) {
 				int vm_size = SystemInformation.VmSize;
+				int vm_rss = SystemInformation.VmRss;
 
-				Logger.Log.Debug ("Memory usage: VmSize={0}  GC.GetTotalMemory={1}",
-						  vm_size, GC.GetTotalMemory (false));
+				Logger.Log.Debug ("Memory usage: VmSize={0:.0} MB, VmRSS={1:.0} MB,  GC.GetTotalMemory={2}",
+						  vm_size/1024.0, vm_rss/1024.0, GC.GetTotalMemory (false));
 
-				if (vm_size > 200 * 1024) {
+				if (vm_size > 300 * 1024) {
 					Logger.Log.Debug ("VmSize too large --- shutting down");
 					Shutdown.BeginShutdown ();
 				}
 
-				Thread.Sleep (1000);
+				Thread.Sleep (5000);
 			}
 		}
 
+
+#if ENABLE_WEBSERVICES
+		static Mono.ASPNET.ApplicationServer appServer = null;
+		static string DEFAULT_XSP_ROOT = Path.Combine (ExternalStringsHack.PkgDataDir, "xsp");
+		static string DEFAULT_XSP_PORT = "8888";
+		//Both "/" and "/beagle" aliased to DEFAULT_XSP_ROOT only for BeagleXSP server
+		static string[] xsp_param = {"--port", DEFAULT_XSP_PORT,
+					     "--root", DEFAULT_XSP_ROOT, 
+					     "--applications", "/:" + DEFAULT_XSP_ROOT + ",/beagle:" + DEFAULT_XSP_ROOT,
+					     "--nonstop"};
+#endif 
 		public static int Main (string[] args)
 		{
 			// Process the command-line arguments
@@ -179,6 +197,14 @@ namespace Beagle.Daemon {
 			int arg_port = 0;
 #endif
 
+#if ENABLE_WEBSERVICES
+			bool web_global = false;
+			bool web_start = false;
+			string web_port = DEFAULT_XSP_PORT;
+			string web_rootDir = DEFAULT_XSP_ROOT;
+			bool web_rootDir_changed = false;
+#endif
+
 			int i = 0;
 			while (i < args.Length) {
 				
@@ -187,6 +213,11 @@ namespace Beagle.Daemon {
 				string next_arg = i < args.Length ? args [i] : null;
 
 				switch (arg) {
+				case "--list-backends":
+					Console.WriteLine ("Current available backends:");
+					Console.Write (QueryDriver.ListBackends ());
+					Environment.Exit (0);
+					break;
 
 				case "--fg":
 				case "--foreground":
@@ -242,6 +273,29 @@ namespace Beagle.Daemon {
 					break;
 #endif
 
+#if ENABLE_WEBSERVICES
+				case "--web-global":
+					web_global = true;
+					break;
+
+				case "--web-start":
+					web_start = true;
+					break;
+
+				case "--web-port":
+					web_port = next_arg;
+					++i; 
+					web_start = true;
+					break;
+
+				case "--web-root":
+					web_rootDir = next_arg;
+					++i; 
+					web_start = true;
+					if (String.Compare(web_rootDir, DEFAULT_XSP_ROOT, true) != 0)
+						web_rootDir_changed = true;
+					break;
+#endif 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
 					break;
@@ -249,6 +303,8 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			MainLoopThread = Thread.CurrentThread;
+
 			// Initialize logging.
 			// If we saw the --debug arg, set the default logging level
 			// accordingly.
@@ -267,6 +323,12 @@ namespace Beagle.Daemon {
 			} catch (Exception ex) { }
 
 
+			if (Environment.UserName == "root") {
+				Logger.Log.Error ("You can not run beagle as root.");
+				Logger.Log.Error ("Beagle is designed to be run from your own user account.");
+				Environment.Exit (-1);
+			}
+
 			// Make sure that extended attributes can be set.  If not, bail out with a message.
 			// FIXME FIXME FIXME: This assumes that EAs work the same on your storage dir
 			// as they do on your home dir, which is obviously not a good assumption.
@@ -289,7 +351,7 @@ namespace Beagle.Daemon {
 			try {
 				Logger.Log.Debug ("Initializing D-BUS");
 				DBusisms.Init ();
-				Application.Init ();
+				Application.InitCheck ("beagled", ref args);
 
 				Logger.Log.Debug ("Acquiring {0} D-BUS service", Beagle.DBusisms.Name);
 				if (!DBusisms.InitService (Beagle.DBusisms.Name)) {
@@ -352,14 +414,12 @@ namespace Beagle.Daemon {
 					Logger.Log.Error ("Could not initialize network service"); 
 				}
 			}
-
 #endif
 
 			// Set up out-of-process indexing
 			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
 				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
 
-
 			// Start the query driver.
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
@@ -373,22 +433,56 @@ namespace Beagle.Daemon {
 			// Start our Inotify threads
 			Logger.Log.Debug ("Starting Inotify threads");
 			Inotify.Start ();
-
+	
 			// Test if the FileAdvise stuff is working: This will print a
 			// warning if not.  The actual advice calls will fail silently.
 			FileAdvise.TestAdvise ();
 
+#if ENABLE_WEBSERVICES		
+			//Beagle Web, WebService access initialization code:
+
+			//start web-access server first
+			Logger.Log.Debug ("Starting WebBackEnd");
+			WebBackEnd.init (web_global);
+
+			//Next start web-service server
+			Logger.Log.Debug ("Starting WebServiceBackEnd");
+			WebServiceBackEnd.init (web_global);
+
+			Logger.Log.Debug ("Global WebAccess {0}", web_global ? "Enabled" : "Disabled");
+
+			xsp_param[1] = web_port;
+			xsp_param[3] = web_rootDir;
+			if (web_rootDir_changed)
+				//Assuming "/beagle" exists as an explicit sub-folder under user specified xsp root directory:
+				xsp_param[5] = "/:" + web_rootDir + ",/beagle:" + web_rootDir + "/beagle";
+			if (web_start) {
+				Logger.Log.Debug ("Starting Internal Web Server");
+				//Start beagled internal web server (BeagleXsp)
+				int retVal = Mono.ASPNET.Server.initXSP(xsp_param, out appServer);
+				if (retVal != 0)
+					Logger.Log.Warn ("Error starting Internal Web Server (retVal={0})", retVal);
+			}
+
+#endif
+
 			Shutdown.ShutdownEvent += OnShutdown;
 
 			stopwatch.Stop ();
 
 			Logger.Log.Debug ("Ready to accept requests after {0}", 
 					 stopwatch);
-			
+		
 			// Start our event loop.
 			Logger.Log.Debug ("Starting main loop");
 			Application.Run ();
 
+#if ENABLE_WEBSERVICES
+			if (appServer != null) {
+			    	appServer.Stop(); 
+				appServer = null;
+			}
+#endif
 			Logger.Log.Debug ("Leaving BeagleDaemon.Main");
 
 			// Exiting will close the dbus connection, which
@@ -398,6 +492,12 @@ namespace Beagle.Daemon {
 
 		private static void OnShutdown ()
 		{
+#if ENABLE_WEBSERVICES
+			if (appServer != null) {
+			    	appServer.Stop(); 
+				appServer = null;
+			}
+#endif
 			// Stop our Inotify threads
 			Inotify.Stop ();
 
