@@ -37,6 +37,7 @@ class ExtractContentTool {
 
 	static bool tokenize = false;
 
+	// FIXME: We don't display structural breaks
 	static void DisplayContent (string line)
 	{
 		if (tokenize) {
@@ -84,7 +85,7 @@ class ExtractContentTool {
 			indexable.Build ();
 
 			Console.WriteLine ("Filename: " + uri);
-			Console.WriteLine ("  Flavor: " + indexable.Flavor);
+			//Console.WriteLine ("  Flavor: " + indexable.Flavor);
 			if (! indexable.HaveFilter)
 				Console.WriteLine ("No filter!");
 
@@ -122,7 +123,7 @@ class ExtractContentTool {
 				else
 					Console.WriteLine ();
 			}
-
+			
 			reader = indexable.GetHotTextReader ();
 			if (reader != null) {
 				string line;
@@ -140,6 +141,7 @@ class ExtractContentTool {
 				else
 					Console.WriteLine ();
 			}
+			
 		}
 	}
 }
